import os

from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceCollectionABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_core.mailing import EMailClientABC, EMailClient
from cpl_discord.discord_event_types_enum import DiscordEventTypesEnum
from cpl_discord.service.discord_collection_abc import DiscordCollectionABC
from flask import Flask

from bot_api.abc.auth_service_abc import AuthServiceABC
from bot_api.api import Api
from bot_api.api_thread import ApiThread
from bot_api.controller.auth_controller import AuthController
from bot_api.controller.auth_discord_controller import AuthDiscordController
from bot_api.controller.discord.server_controller import ServerController
from bot_api.controller.gui_controller import GuiController
from bot_api.event.bot_api_on_ready_event import BotApiOnReadyEvent
from bot_api.service.auth_service import AuthService
from bot_api.service.discord_service import DiscordService
from bot_core.abc.module_abc import ModuleABC
from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum


class ApiModule(ModuleABC):

    def __init__(self, dc: DiscordCollectionABC):
        ModuleABC.__init__(self, dc, FeatureFlagsEnum.api_module)

    def configure_configuration(self, config: ConfigurationABC, env: ApplicationEnvironmentABC):
        cwd = env.working_directory
        env.set_working_directory(os.path.dirname(os.path.realpath(__file__)))
        config.add_json_file(f'config/apisettings.json', optional=False)
        config.add_json_file(f'config/apisettings.{env.environment_name}.json', optional=True)
        config.add_json_file(f'config/apisettings.{env.host_name}.json', optional=True)
        env.set_working_directory(cwd)

    def configure_services(self, services: ServiceCollectionABC, env: ApplicationEnvironmentABC):
        services.add_singleton(EMailClientABC, EMailClient)

        services.add_singleton(ApiThread)
        services.add_singleton(Flask, Api)

        services.add_transient(AuthServiceABC, AuthService)
        services.add_transient(AuthController)
        services.add_transient(AuthDiscordController)
        services.add_transient(GuiController)
        services.add_transient(DiscordService)
        services.add_transient(ServerController)

        # cpl-discord
        self._dc.add_event(DiscordEventTypesEnum.on_ready.value, BotApiOnReadyEvent)
